# Generated by Django 3.0.2 on 2020-10-06 11:55

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Log',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_created=True, auto_now_add=True)),
                ('user_id', models.CharField(max_length=1000)),
                ('category', models.CharField(choices=[('reserve', 'Reserve')], max_length=1000)),
                ('description', models.TextField()),
                ('reference_url', models.URLField(blank=True, null=True)),
            ],
            options={
                'verbose_name': 'log',
                'verbose_name_plural': 'log',
                'ordering': ('-created_at',),
            },
        ),
    ]
